/**
 * @Author Manel Herrera - UOC - PFC
 * @Date 2011/Q4 - V1.0
 *
 * Classe que gestiona els cursos Moodle
 */

package edu.manelherrera.pfc.bus;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Hashtable;
import java.util.Vector;

import net.patrickpollet.moodlews_gson.core.CourseRecord;
import android.content.Context;
import android.content.res.Resources;
import edu.manelherrera.pfc.cla.Category;
import edu.manelherrera.pfc.cla.Course;
import edu.manelherrera.pfc.cla.Session;
import edu.manelherrera.pfc.dat.WSMgr;
import edu.manelherrera.pfc.pre.R;

public class CourseMgr {
	
	private Course course;
	private Session session;
	private Context context;
	private Hashtable<Integer,Category> categories;
	
	public CourseMgr (Session session, Context context) {
		setSession(session);
		setContext(context);
		categories = new Hashtable<Integer,Category>();
	}

	/*
	 * Getter i setters 
	 */
	private void setSession(Session session) {
		this.session = session;
	}

	private void setContext(Context context) {
		this.context = context;
	}
	
	private void setCourse(Course course) {
		this.course = course;
	}
	
	private Course getCourse() {
		return course;
	}

	private Session getSession() {
		return session;
	}

	private Context getContext() {
		return context;
	}
	
	/*
	 * Retorna categories
	 */
	public Hashtable<Integer,Category> getCategories () {
		readCategories();
		return categories;
	}
	
	/*
	 * Creacio d'un curs
	 */
	public String createCourse(Course course) {
		setCourse(course);
		String res = validateCourseData();
		if (res == null) {
			generateRndCourseId();
			WSMgr wsMgr = new WSMgr(getSession());
	    	res = wsMgr.createCourse(getCourse());
	    	if (res != null) {
	    		res = "Moodle error: " + res;
	    	}
		}
		return res;
	}
	
	/*
	 * Modificaci d'un curs
	 */
	public String amendCourse(Course course) {
		setCourse(course);
		String res = validateCourseData();
		if (res == null) {
			WSMgr wsMgr = new WSMgr(getSession());
	    	res = wsMgr.amendCourse(getCourse());
	    	if (res != null) {
	    		res = "Moodle error: " + res;
	    	}
		}
		return res;
	}
	
	/*
	 * Lectura d'un curs
	 */
	public Course readCourse(String course) {
		readCategories();
		WSMgr wsMgr = new WSMgr(getSession());
		CourseRecord[] courseRecord = wsMgr.readCourseByShortName(course);
	    if (courseRecord[0].getError() != null) {
			return null;
		} else {
		    return new Course(courseRecord[0].getId(), courseRecord[0].getIdnumber(), courseRecord[0].getCategory(), 
		    		buildCategory(courseRecord[0].getCategory()), courseRecord[0].getShortname(), 
		    		courseRecord[0].getFullname(), summaryClean(courseRecord[0].getSummary()));
		}
	}
	
	/*
	 * Eliminaci d'un curs
	 */
	public String deleteCourse(Course course) {
		setCourse(course);
		WSMgr wsMgr = new WSMgr(getSession());
	    String res = wsMgr.deleteCourse(getCourse());
	    if (res != null) {
    		res = "Moodle error: " + res;
    	}
	    return res;
	}
	
	/*
	 * Lectura de cursos
	 */
	public Vector<Course> readCourses() {
		readCategories();
		
		WSMgr wsMgr = new WSMgr(getSession());
	    CourseRecord[] courseRecord = wsMgr.readCourses();
		Vector<Course> courses = new Vector<Course>();
		int i = 0;
		while (i < courseRecord.length) {
			if(courseRecord[i].getCategory() != 0) {
				courses.add(new Course(courseRecord[i].getId(), courseRecord[i].getIdnumber(), 
						courseRecord[i].getCategory(), buildCategory(courseRecord[i].getCategory()), 
						courseRecord[i].getShortname(), courseRecord[i].getFullname(), 
						summaryClean(courseRecord[i].getSummary())));
			}
			i++;
		}
		return courses;
	}
	
	/*
	 * Neteja del resum d'un curs per retornar text sense etiquetes
	 */
	private String summaryClean(String summary) {
		StringBuffer sb = new StringBuffer();
		
		int current = 0;
		int start = 0;
		int end = 0;
		int length = summary.length();
				
		while (current < length) {
			start = summary.indexOf("<", current);
			if (start != -1) {
				end = summary.indexOf(">", start);
				if (end == -1) {
					end = length;
				}
			} else {
				start = length;
			}	
			
			sb.append(summary.substring(current, start));	
			current = end + 1;
		}
		return sb.toString();
	}
	
	/*
	 * Llegeix categories de cursos
	 */
	private void readCategories() {
		categories.clear();
		CategoryMgr categoryMgr = new CategoryMgr(getSession());
		categories = categoryMgr.readCategories();
	}
	
	public String buildCategory(int idCategory) {
		Category category = categories.get(idCategory);
		String categoryName = category.getName();
		int categoryParent = category.getParent();
		while (categoryParent != 0) {
			category = categories.get(categoryParent);
			categoryName = category.getName() + "/" + categoryName;
			categoryParent = category.getParent();
		}
		return categoryName;
	}
	
	/*
	 * Validar dades d'un curs creat o modificat
	 */
	private String validateCourseData(){
		Resources r = getContext().getResources();

		if (getCourse().getShortName().compareTo("") == 0) {
			return r.getString(R.string.err_short_blank);
		}
		if (getCourse().getCategory().compareTo("") == 0) {
			return r.getString(R.string.err_category_blank);
		}
		if (getCourse().getFullName().compareTo("") == 0) {
			return r.getString(R.string.err_full_blank);
		}
		if (getCourse().getSummary().compareTo("") == 0) {
			return r.getString(R.string.err_summary_blank);
		}
		else return null;
	}
	
	/*
	 * Genera id de curs aleatori
	 */
	private void generateRndCourseId() {
		Calendar c = new GregorianCalendar();
		
		StringBuffer sb = new StringBuffer();
		sb.append(getCourse().getShortName().concat("$$").substring(0, 3).trim());
		sb.append(Integer.toString(c.get(Calendar.YEAR)));
		sb.append(Integer.toString(c.get(Calendar.MONTH)));
		sb.append(Integer.toString(c.get(Calendar.DATE)));
		sb.append(Integer.toString(c.get(Calendar.HOUR_OF_DAY)));
		sb.append(Integer.toString(c.get(Calendar.MINUTE)));
		sb.append(Integer.toString(c.get(Calendar.SECOND)));
		sb.append(Integer.toString(c.get(Calendar.MILLISECOND)));
		
		getCourse().setIdNumber(sb.toString());
	}
}
