/**
 * @Author Manel Herrera - UOC - PFC
 * @Date 2011/Q4 - V1.0
 *
 * Classe que gestiona les matrcules Moodle
 */

package edu.manelherrera.pfc.bus;

import java.util.Hashtable;
import java.util.Vector;

import net.patrickpollet.moodlews_gson.core.CourseRecord;
import android.content.Context;
import android.content.res.Resources;
import edu.manelherrera.pfc.cla.Category;
import edu.manelherrera.pfc.cla.Course;
import edu.manelherrera.pfc.cla.Session;
import edu.manelherrera.pfc.cla.User;
import edu.manelherrera.pfc.dat.WSMgr;
import edu.manelherrera.pfc.pre.R;

public class RoleMgr {
	
	private Session session;
	private Course course;
	private Context context;
	private Hashtable<Integer,Category> categories;
	
	public RoleMgr (Session session, Context context) {
		setSession(session);
		setContext(context);
		categories = new Hashtable<Integer,Category>();
	}

	/*
	 * Getters i setters
	 */
	private void setSession(Session session) {
		this.session = session;
	}

	private Session getSession() {
		return session;
	}
	
	private void setCourse(Course course) {
		this.course = course;
	}
	
	private Course getCourse() {
		return course;
	}
	
	private void setContext(Context context) {
		this.context = context;
	}

	private Context getContext() {
		return context;
	}
	
	/*  
	 * Retorna categories
	 */
	public Hashtable<Integer,Category> getCategories () {
		readCategories();
		return categories;
	}
	
	/*
	 * Crea matrcula
	 */
	public String createEnrolment(Course course, User user) {
		setCourse(course);
		String res = validateRoleData();
		if (res == null) {
			String[] usr = new String[1];
			usr[0] = user.getuserName();
			WSMgr wsMgr = new WSMgr(getSession());
			res = wsMgr.createEnrolment(course.getShortName(), usr);
	    	if (res.compareTo("err") == 0) {
	    		res = "Moodle error: " + getContext().getResources().getString(R.string.err_permissions);
	    	} else if (res.compareTo("") != 0) {
	    		res = "Moodle error: " + res;
	    	}
		}
		return res;
	}
	
	/*
	 * Elimina matricula
	 */
	public String deleteEnrolment(Course course, User user) {
		setCourse(course);
		String res = validateRoleData();
		if (res == null) {
			String[] usr = new String[1];
			usr[0] = user.getuserName();
			WSMgr wsMgr = new WSMgr(getSession());
			res = wsMgr.deleteEnrolment(course.getShortName(), usr);
	    	if (res.compareTo("err") == 0) {
	    		res = "Moodle error: " + getContext().getResources().getString(R.string.err_permissions);
	    	} else if (res.compareTo("") != 0) {
	    		res = "Moodle error: " + res;
	    	}
		}
		return res;
	}
	
	/*
	 * Llegeis matriculacions
	 */
	public Vector<Course> readEnrolmentsByUserName(String usr) {
		readCategories();
		
		WSMgr wsMgr = new WSMgr(getSession());
	    CourseRecord[] courseRecord = wsMgr.getEnrolmentsByUserName(usr);
		Vector<Course> courses = new Vector<Course>();
		int i = 0;
		while (i < courseRecord.length) {
			if(courseRecord[i].getCategory() != 0) {
				courses.add(new Course(courseRecord[i].getId(), courseRecord[i].getIdnumber(), 
						courseRecord[i].getCategory(), buildCategory(courseRecord[i].getCategory()), 
						courseRecord[i].getShortname(), courseRecord[i].getFullname(), 
						summaryClean(courseRecord[i].getSummary())));
			}
			i++;
		}
		return courses;
	}
	
	/*
	 * Neteja del resum d'un curs per retornar text sense etiquetes
	 */
	private String summaryClean(String summary) {
		StringBuffer sb = new StringBuffer();
		
		int current = 0;
		int start = 0;
		int end = 0;
		int length = summary.length();
				
		while (current < length) {
			start = summary.indexOf("<", current);
			if (start != -1) {
				end = summary.indexOf(">", start);
				if (end == -1) {
					end = length;
				}
			} else {
				start = length;
			}	
			
			sb.append(summary.substring(current, start));	
			current = end + 1;
		}
		return sb.toString();
	}
	
	/*
	 * LLegeix categories
	 */
	private void readCategories() {
		categories.clear();
		CategoryMgr categoryMgr = new CategoryMgr(getSession());
		categories = categoryMgr.readCategories();
	}
	
	/*
	 * Monta categoria 
	 */
	public String buildCategory(int idCategory) {
		Category category = categories.get(idCategory);
		String categoryName = category.getName();
		int categoryParent = category.getParent();
		while (categoryParent != 0) {
			category = categories.get(categoryParent);
			categoryName = category.getName() + "/" + categoryName;
			categoryParent = category.getParent();
		}
		return categoryName;
	}
	
	/*
	 * Valida dades matrcula
	 */
	private String validateRoleData(){
		Resources r = getContext().getResources();

		if (getCourse() == null) {
			return r.getString(R.string.err_course_blank);
		}
		if (getCourse().getShortName().compareTo("") == 0) {
			return r.getString(R.string.err_course_blank);
		}
		else return null;
	}
	
}
