/**
 * @Author Manel Herrera - UOC - PFC
 * @Date 2011/Q4 - V1.0
 *
 * Classe que gestiona els usuaris Moodle
 */

package edu.manelherrera.pfc.bus;

import java.util.Calendar;
import java.util.GregorianCalendar;

import android.content.Context;
import android.content.res.Resources;

import net.patrickpollet.moodlews_gson.core.UserRecord;

import edu.manelherrera.pfc.cla.Session;
import edu.manelherrera.pfc.cla.User;
import edu.manelherrera.pfc.dat.WSMgr;
import edu.manelherrera.pfc.pre.R;

public class UserMgr {
	
	private User user;
	private Session session;
	private Context context;
	
	public UserMgr (Session session, Context context) {
		setSession(session);
		setContext(context);
	}

	/*
	 * Getters i setters
	 */
	private void setSession(Session session) {
		this.session = session;
	}

	private void setContext(Context context) {
		this.context = context;
	}
	
	private void setUser(User user) {
		this.user = user;
	}
	
	private User getUser() {
		return user;
	}

	private Session getSession() {
		return session;
	}

	private Context getContext() {
		return context;
	}

	/*
	 * Crea usuari
	 */
	public String createUser(User user) {
		setUser(user);
		String res = validateUserData(true);
		if (res == null) {
			generateRndUserId();
			WSMgr wsMgr = new WSMgr(getSession());
	    	res = wsMgr.createUser(getUser());
	    	if (res != null) {
	    		res = "Moodle error: " + res;
	    	}
		}
		return res;
	}
	
	/*
	 * Modifica usuari
	 */
	public String amendUser(User user) {
		setUser(user);
		String res = validateUserData(false);
		if (res == null) {
			WSMgr wsMgr = new WSMgr(getSession());
	    	res = wsMgr.amendUser(getUser());
	    	if (res != null) {
	    		res = "Moodle error: " + res;
	    	}
		}
		return res;
	}
	
	/*
	 * Llegeix usuari
	 */
	public User readUser(String usr) {
		WSMgr wsMgr = new WSMgr(getSession());
	    UserRecord[] userRecord = wsMgr.readUserByUserName(usr);
	    if (userRecord[0].getError() != null) {
			return null;
		} else {
		    return new User(usr, "" , userRecord[0].getFirstname(), 
					userRecord[0].getLastname(), userRecord[0].getEmail());
		}
	}
	
	/*
	 * Elimina usuari
	 */
	public String deleteUser(User user) {
		setUser(user);
		WSMgr wsMgr = new WSMgr(getSession());
	    String res = wsMgr.deleteUser(getUser());
	    if (res != null) {
    		res = "Moodle error: " + res;
    	}
	    return res;
	}
	
	/*
	 * Valida dades usuari
	 */
	private String validateUserData(boolean pwdCheck){
		Resources r = getContext().getResources();

		if (getUser().getuserName().compareTo("") == 0) {
			return r.getString(R.string.err_usr_blank);
		}
		if (pwdCheck && getUser().getpassword().length() < 8) {
			return r.getString(R.string.err_pwd_blank);
		}
		if (getUser().getfirstName().compareTo("") == 0) {
			return r.getString(R.string.err_name_blank);
		}
		if (getUser().getlastName().compareTo("") == 0) {
			return r.getString(R.string.err_surname_blank);
		}
		if (getUser().geteMail().compareTo("") == 0) {
			return r.getString(R.string.err_e_mail_blank);
		}
		else return null;
	}
	
	/*
	 * Genera id usuari aleatori
	 */
	private void generateRndUserId() {
		Calendar c = new GregorianCalendar();
		
		StringBuffer sb = new StringBuffer();
		sb.append(getUser().getfirstName().substring(0, 1));
		sb.append(getUser().getlastName().substring(0, 1));
		sb.append(Integer.toString(c.get(Calendar.YEAR)));
		sb.append(Integer.toString(c.get(Calendar.MONTH)));
		sb.append(Integer.toString(c.get(Calendar.DATE)));
		sb.append(Integer.toString(c.get(Calendar.HOUR_OF_DAY)));
		sb.append(Integer.toString(c.get(Calendar.MINUTE)));
		sb.append(Integer.toString(c.get(Calendar.SECOND)));
		sb.append(Integer.toString(c.get(Calendar.MILLISECOND)));
		
		getUser().setIdNumber(sb.toString());
	}
}
