/**
 * @Author Manel Herrera - UOC - PFC
 * @Date 2011/Q4 - V1.0
 *
 * Classe que gestiona l'accs Web Service Moodle
 */

package edu.manelherrera.pfc.dat;

import net.patrickpollet.moodlews_gson.core.CategoryRecord;
import net.patrickpollet.moodlews_gson.core.CourseDatum;
import net.patrickpollet.moodlews_gson.core.CourseRecord;
import net.patrickpollet.moodlews_gson.core.EnrolRecord;
import net.patrickpollet.moodlews_gson.core.LoginReturn;
import net.patrickpollet.moodlews_gson.core.Mdl_restserverBindingStub;
import net.patrickpollet.moodlews_gson.core.UserDatum;
import net.patrickpollet.moodlews_gson.core.UserRecord;
import edu.manelherrera.pfc.cla.Course;
import edu.manelherrera.pfc.cla.Session;
import edu.manelherrera.pfc.cla.User;

public class WSMgr {

	private static final String MOODLE_SERVICE = "wspp/service_pp2.php";
	private static final String MOODLE_NAMESPACE = "json"; 
	
	private Session session;
	private String url; 
	private Mdl_restserverBindingStub moodle = null;
	private LoginReturn lr = null;
	
	public WSMgr(String url, String usr, String pwd, String token) {
		setUrl(url);
		login(usr, pwd, token);
	}
	
	public WSMgr(Session session) {
		setSession(session);
	}
	
	/*
	 * Getters i setters
	 */
	private void setUrl(String url) {
		this.url = url;
	}
	
	private String getUrl() {
		return url;
	}
	
	private void setSession(Session session) {
		this.session = session;	
	}
		
	private void setSession(String url, String key, int client, int userId, String usr, String userDesc) {
		this.session = new Session(url, key, client, userId, usr, userDesc);	
	}
	
	public Session getSession() {
		return session;
	}
	
	/*
	 * Inici sessio
	 */
	private void login(String usr, String pwd, String token) {
				
		moodle = new Mdl_restserverBindingStub(getUrl() + MOODLE_SERVICE, MOODLE_NAMESPACE, false);
		lr = moodle.login(usr, pwd);

		if (lr != null) {
			String key = lr.getSessionkey();
			int client = lr.getClient();
			int userId = moodle.get_my_id(client, key);
			
			UserRecord[] res = moodle.get_user_byid(client, key, userId);
			String userDesc = res[0].getFirstname() + " " + res[0].getLastname();
			
			setSession(url, key, client, userId, usr, userDesc);
		} else {
			session = null;
		}
	}
	
	/*
	 * Fi Sessio
	 */
	public boolean logout(Session session) {
		moodle = new Mdl_restserverBindingStub(session.getUrl() + MOODLE_SERVICE, MOODLE_NAMESPACE, false);
		return moodle.logout(session.getClient(), session.getKey());
	}
	
	/*
	 * Crea usuari
	 */
	public String createUser(User user) {
		UserDatum userDat = buildUserDatum(user);
		moodle = new Mdl_restserverBindingStub(session.getUrl() + MOODLE_SERVICE, MOODLE_NAMESPACE, false);
		UserRecord[] res = moodle.add_user(session.getClient(), 
				session.getKey(), userDat);
		return res[0].getError();
	}
	
	/* 
	 * Modifica usuari
	 */
	public String amendUser(User user) {
		UserDatum[] userDat = new UserDatum[1];
		userDat[0] = buildUserDatum(user);
		userDat[0].setAction("update");
		userDat[0].setPassword(null);
	
		moodle = new Mdl_restserverBindingStub(session.getUrl() + MOODLE_SERVICE, MOODLE_NAMESPACE, false);
		UserRecord[] res = moodle.update_user(session.getClient(), 
				session.getKey(), userDat[0], "username");
		
		return res[0].getError();
	}
	
	/*
	 * Elimina usuari
	 */
	public String deleteUser(User user) {
		moodle = new Mdl_restserverBindingStub(session.getUrl() + MOODLE_SERVICE, MOODLE_NAMESPACE, false);
		UserRecord[] res = moodle.delete_user(session.getClient(), session.getKey(), user.getuserName(), "username");
		return res[0].getError();
	}
	
	/* 
	 * Llegeix usuari per nom
	 */
	public UserRecord[] readUserByUserName(String usr) {
		moodle = new Mdl_restserverBindingStub(session.getUrl() + MOODLE_SERVICE, MOODLE_NAMESPACE, false);
		return moodle.get_user_byusername(session.getClient(), session.getKey(), usr);
	}
	
	/* 
	 * Crear curs
	 */
	public String createCourse(Course course) {
		CourseDatum courseDat = buildCourseDatum(course);
		moodle = new Mdl_restserverBindingStub(session.getUrl() + MOODLE_SERVICE, MOODLE_NAMESPACE, false);
		CourseRecord[] res = moodle.add_course(session.getClient(), 
				session.getKey(), courseDat);
		return res[0].getError();
	}
	
	/*
	 * Modificar curs
	 */
	public String amendCourse(Course course) {
		CourseDatum[] courseDat = new CourseDatum[1];
		courseDat[0] = buildCourseDatum(course);
		courseDat[0].setAction("update");
	
		moodle = new Mdl_restserverBindingStub(session.getUrl() + MOODLE_SERVICE, MOODLE_NAMESPACE, false);
		CourseRecord[] res = moodle.update_course(session.getClient(), 
				session.getKey(), courseDat[0], "id");
		
		return res[0].getError();
	}
	
	/*
	 * Eliminar curs
	 */
	public String deleteCourse(Course course) {
		moodle = new Mdl_restserverBindingStub(session.getUrl() + MOODLE_SERVICE, MOODLE_NAMESPACE, false);
		CourseRecord[] res = moodle.delete_course(session.getClient(), session.getKey(), course.getShortName(), "shortname");
		return res[0].getError();
	}
	
	/*
	 * LLegir curs per nom curt
	 */
	public CourseRecord[] readCourseByShortName(String course) {
		moodle = new Mdl_restserverBindingStub(session.getUrl() + MOODLE_SERVICE, MOODLE_NAMESPACE, false);
		return moodle.get_course(session.getClient(), session.getKey(), course, "shortname");
	}
	
	/*
	 * Llegir tots els cursos
	 */
	public CourseRecord[] readCourses() {
		moodle = new Mdl_restserverBindingStub(session.getUrl() + MOODLE_SERVICE, MOODLE_NAMESPACE, false);
		String[] str = null;
		return moodle.get_courses(session.getClient(), session.getKey(), str, "idnumber");
	}

	/*
	 * Llegir categories de cursos
	 */
	public CategoryRecord[] readCategories() {
		moodle = new Mdl_restserverBindingStub(session.getUrl() + MOODLE_SERVICE, MOODLE_NAMESPACE, false);
		return moodle.get_categories(session.getClient(), session.getKey(), "1", "visible");
	}
	
	/*
	 * Crear matricula
	 */
	public String createEnrolment(String course, String[] usr) {
		moodle = new Mdl_restserverBindingStub(session.getUrl() + MOODLE_SERVICE, MOODLE_NAMESPACE, false);
		EnrolRecord[] res = moodle.enrol_students(session.getClient(), session.getKey(), course, "shortname", usr, "username");
		if (res == null) {
			return "err";
		} else {
			return res[0].getError();
		}
	}

	/*
	 * Eliminar matrcula
	 */
	public String deleteEnrolment(String course, String[] usr) {
		moodle = new Mdl_restserverBindingStub(session.getUrl() + MOODLE_SERVICE, MOODLE_NAMESPACE, false);
		EnrolRecord[] res = moodle.unenrol_students(session.getClient(), session.getKey(), course, "shortname", usr, "username");
		if (res == null) {
			return "err";
		} else {
			return res[0].getError();
		}
	}
	
	/*
	 * LLegir matrcules per nom d'usuari
	 */
	public CourseRecord[] getEnrolmentsByUserName(String usr) {
		moodle = new Mdl_restserverBindingStub(session.getUrl() + MOODLE_SERVICE, MOODLE_NAMESPACE, false);
		return moodle.get_my_courses_byusername(session.getClient(), session.getKey(), usr, "");
	}
	
	/*
	 * Crear estructura d'Usuari
	 */
	private UserDatum buildUserDatum(User user) {
		UserDatum userDat = new UserDatum();
		userDat.setUsername(user.getuserName());
		if (user.getpassword().compareTo("") != 0) userDat.setPassword(user.getpassword());
		userDat.setFirstname(user.getfirstName());
		userDat.setLastname(user.getlastName());
		userDat.setIdnumber(user.getidNumber());
		userDat.setEmail(user.geteMail());
		
		return userDat;
	}
	
	/*
	 * Crear estructura de curs
	 */
	private CourseDatum buildCourseDatum(Course course) {
		CourseDatum courseDat = new CourseDatum();
		courseDat.setId(course.getId());
		courseDat.setCategory(course.getCategoryId());
		courseDat.setShortname(course.getShortName());
		courseDat.setFullname(course.getFullName());
		courseDat.setSummary(course.getSummary());
		courseDat.setIdnumber(course.getIdNumber());
		
		return courseDat;
	}
	
}
