/**
 * @Author Manel Herrera - UOC - PFC
 * @Date 2011/Q4 - V1.0
 *
 * Classe que interacciona amb modifiacci de cursos
 */

package edu.manelherrera.pfc.pre;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemSelectedListener;
import edu.manelherrera.pfc.bus.CourseMgr;
import edu.manelherrera.pfc.cla.Category;
import edu.manelherrera.pfc.cla.Course;
import edu.manelherrera.pfc.cla.Session;

public class CourseAmendAct extends Activity implements OnClickListener, 
		OnItemSelectedListener{
	
	//Declaracions
	 private static final int INITIAL = 0;
	 private static final int BODY = 1;
	 
	 private int status;
	 
	 private Course course;
	 private CourseMgr courseMgr;
	 
	 private TextView label_usrDesc; 
	 private Spinner spin_cat;
	 private Spinner spin_short; 
	 private EditText edit_short; 
	 private EditText edit_full; 
	 private EditText edit_summary; 
	 private String[] categories;
	 private int catSel;
	 
	 private ImageButton button_del;
	 private ImageButton button_close;
	 private ImageButton button_save;
	 private LinearLayout layout_body;
	 
	 private Hashtable<String,Course> coursesById;
	 private Hashtable<String,String> coursesByName;
	 private Hashtable<Integer,Category> categoriesById;
	 private Hashtable<String,Integer> categoriesByName;
	 
	 private ArrayAdapter<String> aAdapterShort;
	 private ArrayAdapter<String> aAdapterCat;
	 private List<String> courses;
	 
	 private Session session;
	
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.course_amend);
        
        //Inicialitzacions
        coursesById = new Hashtable<String,Course>();
        coursesByName = new Hashtable<String,String>();
		categoriesById = new Hashtable<Integer,Category>();
		categoriesByName = new Hashtable<String, Integer>();
        
        spin_cat = (Spinner)findViewById(R.id.spin_cat);
        spin_short = (Spinner)findViewById(R.id.spin_short); 
        edit_short = (EditText)findViewById(R.id.edit_short);
        edit_full = (EditText)findViewById(R.id.edit_full); 
        edit_summary = (EditText)findViewById(R.id.edit_summary); 
        label_usrDesc = (TextView) findViewById(R.id.label_usrname);
        button_save = (ImageButton) findViewById(R.id.button_save);
        button_del = (ImageButton) findViewById(R.id.button_del);
        button_close = (ImageButton) findViewById(R.id.button_close);
        layout_body = (LinearLayout) findViewById(R.id.layout_body);
      
        button_del.setOnClickListener(this);
        button_close.setOnClickListener(this);
        button_save.setOnClickListener(this);
        spin_cat.setOnItemSelectedListener(this);
        spin_short.setOnItemSelectedListener(this);
        
        Bundle b = getIntent().getExtras();
        session = (Session) b.getParcelable("session"); 
     
        label_usrDesc.setText(session.getUserDesc());
        
        //Llegeix cursos i categories
        courses = new ArrayList<String>();
        aAdapterShort = new ArrayAdapter<String>(this,
				R.layout.layout_spinner, courses);
        aAdapterShort.setDropDownViewResource(R.layout.simple_spinner_dropdown_item);
		spin_short.setAdapter(aAdapterShort);
		
        categories = readCategories();
        aAdapterCat = new ArrayAdapter<String>(this,
				R.layout.layout_spinner, categories);
        aAdapterCat.setDropDownViewResource(R.layout.simple_spinner_dropdown_item);
		spin_cat.setAdapter(aAdapterCat);
		
		changeStatus(INITIAL);
    }
    
    /*
     * Si es clica un bot
     */
    @Override
    public void onClick(View v) {
    	
    	switch (v.getId()) {
		//Salvar
		case R.id.button_save:
	    	int catId = -1;
	    	String catName = "";
	    	if (catSel != 0) {
	    		String catQualifiedName = categories[catSel];
	    		catId = categoriesByName.get(catQualifiedName);
	        	catName = categoriesById.get(catId).getName();
	    	}
	    	String short_name = edit_short.getText().toString().trim();
			String full_name = edit_full.getText().toString().trim();
			String summary = edit_summary.getText().toString().trim();

			Course courseNew = new Course(course.getId(), course.getIdNumber(), catId, catName,
					short_name, full_name, "<p>" + summary + "</p>");
			CourseMgr courseMgr = new CourseMgr(session, this);
			String res = courseMgr.amendCourse(courseNew);
			if (res == null) {
				Toast.makeText(this, getString(R.string.course_updated), Toast.LENGTH_LONG).show();
				changeStatus(INITIAL);
			} else {
				errMsg(res);
			}
			break;
			
		//Tancar
		case R.id.button_close:
			switch (status) {
			case INITIAL:
				this.finish();
				break;
			case BODY:
				changeStatus(INITIAL);
				break;
			}
			break;
			
		//Eliminar
		case R.id.button_del:
			AlertDialog.Builder alertbox = new AlertDialog.Builder(this);
			alertbox.setCancelable(false);
			alertbox.setMessage(getString(R.string.course_delete_conf));
			alertbox.setPositiveButton(R.string.yes, new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface arg0, int arg1) {
					delete();
				}
			});alertbox.setNegativeButton(R.string.no, new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog, int arg1) {
					dialog.cancel();
				}
			});
			
			alertbox.show();
			
			break;
		}
	}
     
    /*
     * Si se selecciona un element de la llista
     */
	@Override
	public void onItemSelected(AdapterView<?> parent, View v, int position, long id) {
		switch(parent.getId()) {
		case R.id.spin_short:
			if ((Integer)position != 0) {
				courseMgr = new CourseMgr(session, this);
				course = courseMgr.readCourse(courses.get(position));

				if (course == null) {
					errMsg(getString(R.string.err_course_no_exist));
					spin_cat.setSelection(0);
				} else {
					edit_short.setText(course.getShortName());
					edit_full.setText(course.getFullName());
					edit_summary.setText(course.getSummary());
					String qualifiedName = course.getCategory();
					for (int i = 0; i <= (categoriesById.size()); i++) {
						if (qualifiedName.compareTo(categories[i]) == 0) {
							spin_cat.setSelection(i);
							i = categoriesById.size();
						}
					}
					changeStatus(BODY);
				}
			}
			break;
			
		case R.id.spin_cat:
			catSel = position;
			break;
		}
		
	}
	
	@Override
	public void onNothingSelected(AdapterView<?> arg0) {
		catSel = 0;
	}
	
	/*
	 * Llegir cursos
	 */
	private void readCourses() {
    	coursesById.clear();
    	coursesByName.clear();
    	courses.clear();
    	
    	CourseMgr courseMgr = new CourseMgr(session, this);
    	Vector<Course> coursesV = courseMgr.readCourses();

    	Iterator<Course> itr = coursesV.iterator();
    	while (itr.hasNext()) {
          	Course course = itr.next();
          	coursesById.put(course.getIdNumber(), course);
          	coursesByName.put(course.getShortName(), course.getIdNumber());
        	courses.add(course.getShortName());
    	}
    	courses.add("");
    	Collections.sort(courses);
    }
	
	/*
	 * Llegir categories
	 */
	private String[] readCategories() {
    	categoriesById.clear();
    	categoriesByName.clear();
    	int i = 0;
    	
    	CourseMgr courseMgr = new CourseMgr(session, this);
    	categoriesById = courseMgr.getCategories();
    	String[] categories = new String [categoriesById.size() + 1];
    	
    	Enumeration<Integer> e = categoriesById.keys();
    	while (e.hasMoreElements()) {
    		int catId = e.nextElement();
        	String catQualifiedName = courseMgr.buildCategory(catId);
        	categoriesByName.put(catQualifiedName, catId);
        	categories[i] = catQualifiedName;
        	i++;
    	}
    	categories[i] = "";
    	java.util.Arrays.sort(categories);
    	return categories;
    }
    
	/*
	 * Mostrar error
	 */
    private void errMsg(String msg) {
    	AlertDialog.Builder alertbox = new AlertDialog.Builder(this);
		alertbox.setMessage(msg);
		alertbox.setNeutralButton(R.string.ok, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface arg0, int arg1) {}
		});
		alertbox.show();
    }
    
    /*
     * Eliminar
     */
    private void delete() {
    	String sName = edit_short.getText().toString().trim();
	    String fName = edit_full.getText().toString().trim();
	    String sum = edit_summary.getText().toString().trim();
	        
	   	Course course = new Course(0, "", sName, fName, sum);
	   	courseMgr = new CourseMgr(session, this);
	   	String res = courseMgr.deleteCourse(course);
	   	if (res == null) {
	   		Toast.makeText(this, getString(R.string.course_deleted), Toast.LENGTH_LONG).show();
	   		changeStatus(INITIAL);
	   	} else {
	   		errMsg(res);
	   	}
    }
    
    /*
     * Neteja de camps
     */
    private void cleanFields(){
    	spin_cat.setSelection(0);
    	spin_short.setSelection(0);
    	edit_short.setText("");
    	edit_full.setText("");
    	edit_summary.setText("");
    	edit_short.requestFocus();
    }
    
    /*
     * Canvi estat pantalla
     */
    private void changeStatus(int status) {
    	this.status = status;
    	switch (status) {
		case INITIAL:
			readCourses();
			aAdapterShort.notifyDataSetChanged();
			cleanFields();
    		layout_body.setVisibility(View.GONE);
            button_del.setVisibility(View.GONE);
            spin_short.setVisibility(View.VISIBLE);
            break;
		case BODY:
      		layout_body.setVisibility(View.VISIBLE);
            button_del.setVisibility(View.VISIBLE);
            spin_short.setVisibility(View.GONE);
            edit_short.requestFocus();
            break;
    	}
    }
}