/**
 * @Author Manel Herrera - UOC - PFC
 * @Date 2011/Q4 - V1.0
 *
 * Classe que interacciona amb creacio de cursos
 */

package edu.manelherrera.pfc.pre;

import java.util.Enumeration;
import java.util.Hashtable;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemSelectedListener;
import edu.manelherrera.pfc.bus.CourseMgr;
import edu.manelherrera.pfc.cla.Category;
import edu.manelherrera.pfc.cla.Course;
import edu.manelherrera.pfc.cla.Session;

public class CourseCreateAct extends Activity implements OnClickListener, 
			OnItemSelectedListener{
	
	//Declaracions
	 private TextView label_usrDesc; 
	 private Spinner spin_cat; 
	 private EditText edit_short; 
	 private EditText edit_full; 
	 private EditText edit_summary; 
	 private ImageButton button_save;	
	 private String[] categories;
	 private int catSel;
	 
	 private Hashtable<Integer,Category> categoriesById;
	 private Hashtable<String,Integer> categoriesByName;
	 
	 private Session session;
	
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.course_create);
        
        //Inicialitzacions
        categoriesById = new Hashtable<Integer,Category>();
		categoriesByName = new Hashtable<String, Integer>();
		
        spin_cat = (Spinner)findViewById(R.id.spin_cat); 
        edit_short = (EditText)findViewById(R.id.edit_short);
        edit_full = (EditText)findViewById(R.id.edit_full); 
        edit_summary = (EditText)findViewById(R.id.edit_summary); 
        label_usrDesc = (TextView) findViewById(R.id.label_usrname);
        button_save = (ImageButton) findViewById(R.id.button_save);
      
        button_save.setOnClickListener(this);	
        spin_cat.setOnItemSelectedListener(this);
        
        Bundle b = getIntent().getExtras();
        session = (Session) b.getParcelable("session"); 
     
        label_usrDesc.setText(session.getUserDesc());

        categories = readCategories();
        ArrayAdapter<String> aAdapter = new ArrayAdapter<String>(this,
				R.layout.layout_spinner, categories);
        aAdapter.setDropDownViewResource(R.layout.simple_spinner_dropdown_item);
		spin_cat.setAdapter(aAdapter);
    }
    
    /*
     * Si es clica un bot
     */
    @Override
    public void onClick(View v) {
    	int catId = -1;
    	String catName = "";
    	if (catSel != 0) {
    		String catQualifiedName = categories[catSel];
    		catId = categoriesByName.get(catQualifiedName);
        	catName = categoriesById.get(catId).getName();
    	}
    	String short_name = edit_short.getText().toString().trim();
		String full_name = edit_full.getText().toString().trim();
		String summary = edit_summary.getText().toString().trim();

		Course course = new Course(catId, catName, short_name, full_name, 
				"<p>" + summary + "</p>");
		CourseMgr courseMgr = new CourseMgr(session, this);
		String res = courseMgr.createCourse(course);
		if (res == null) {
			
			Toast.makeText(this, getString(R.string.course_created), Toast.LENGTH_LONG).show();
			cleanFields();
		} else {
			errMsg(res);
		} 
	}
       
    /*
     * Si se selecciona un element de la llista
     */
	@Override
	public void onItemSelected(AdapterView<?> parent, View v, int position,
			long id) {
		catSel = position;
	}
	
	@Override
	public void onNothingSelected(AdapterView<?> arg0) {
		catSel = 0;
	}
	
	/*
	 * mostra error
	 */
    private void errMsg(String msg) {
    	AlertDialog.Builder alertbox = new AlertDialog.Builder(this);
		alertbox.setMessage(msg);
		alertbox.setNeutralButton(getString(R.string.ok), new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface arg0, int arg1) {}
		});
		alertbox.show();
    }
    
    /*
     * Neteja camps
     */
    private void cleanFields(){
    	spin_cat.setSelection(0);
    	edit_short.setText("");
    	edit_full.setText("");
    	edit_summary.setText("");
    	edit_short.requestFocus();
    }
    
    /*
     * LLegeix categories
     */
    private String[] readCategories() {
    	categoriesById.clear();
    	categoriesByName.clear();
    	int i = 0;
    	
    	CourseMgr courseMgr = new CourseMgr(session, this);
    	categoriesById = courseMgr.getCategories();
    	String[] categories = new String [categoriesById.size() + 1];
    	
    	Enumeration<Integer> e = categoriesById.keys();
    	while (e.hasMoreElements()) {
    		int catId = e.nextElement();
        	String catQualifiedName = courseMgr.buildCategory(catId);
        	categoriesByName.put(catQualifiedName, catId);
        	categories[i] = catQualifiedName;
        	i++;
    	}
    	categories[i] = "";
    	java.util.Arrays.sort(categories);
    	return categories;
    }
} 