/**
 * @Author Manel Herrera - UOC - PFC
 * @Date 2011/Q4 - V1.0
 *
 * Classe que interacciona amb la fitxa d'un curs
 */


package edu.manelherrera.pfc.pre;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.text.method.ScrollingMovementMethod;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.AdapterView.OnItemSelectedListener;
import edu.manelherrera.pfc.bus.CourseMgr;
import edu.manelherrera.pfc.cla.Course;
import edu.manelherrera.pfc.cla.Session;

public class CourseDetailAct extends Activity implements OnClickListener, 
		OnItemSelectedListener{
	
	//Declaracions
	 private static final int INITIAL = 0;
	 private static final int BODY = 1;
	 
	 private int status;
	 private boolean comeFromListView = false;
	 
	 private Course course;
	 private CourseMgr courseMgr;
	 
	 private TextView label_usrDesc; 
	 private Spinner spin_short; 
	 private TextView edit_short;
	 private TextView edit_cat; 
	 private TextView edit_full; 
	 private TextView edit_summary; 
	 
	 private ImageButton button_close;
	 private LinearLayout layout_body;
	 
	 private Hashtable<String,Course> coursesById;
	 private Hashtable<String,String> coursesByName;
	 
	 private ArrayAdapter<String> aAdapterShort;
	 private List<String> courses;
	 
	 private Session session;
	
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.course_detail);
        
        //Inicialitzacions
        coursesById = new Hashtable<String,Course>();
        coursesByName = new Hashtable<String,String>();
        
        spin_short = (Spinner)findViewById(R.id.spin_short); 
        edit_short = (TextView)findViewById(R.id.edit_short);
        edit_cat = (TextView)findViewById(R.id.edit_cat);
        edit_full = (TextView)findViewById(R.id.edit_full); 
        edit_summary = (TextView)findViewById(R.id.edit_summary); 
        label_usrDesc = (TextView) findViewById(R.id.label_usrname);
        button_close = (ImageButton) findViewById(R.id.button_close);
        layout_body = (LinearLayout) findViewById(R.id.layout_body);
      
        button_close.setOnClickListener(this);
        spin_short.setOnItemSelectedListener(this);
        
        edit_summary.setMovementMethod(new ScrollingMovementMethod());
        
        Bundle b = getIntent().getExtras();
        session = (Session) b.getParcelable("session");
        course = (Course) b.getParcelable("course"); 
     
        label_usrDesc.setText(session.getUserDesc());
        
        //Llgeix cursos
        courses = new ArrayList<String>();
        aAdapterShort = new ArrayAdapter<String>(this,
				R.layout.layout_spinner, courses);
        aAdapterShort.setDropDownViewResource(R.layout.simple_spinner_dropdown_item);
		spin_short.setAdapter(aAdapterShort);
		
		if (course == null) { 
			comeFromListView = false;
			changeStatus(INITIAL);
		} else {
			comeFromListView = true;
			loadDetail();
		}
    }
    
    /*
     * Si es clica un bot
     */
    @Override
    public void onClick(View v) {
		
    	switch (status) {
		case INITIAL:
			this.finish();
			break;
		case BODY:
			if (comeFromListView) {
				this.finish(); 
			} else {
				changeStatus(INITIAL);	
			}
			break;
		}
	}
     
    /*
     * Si se selecciona un element de la llista
      */
	@Override
	public void onItemSelected(AdapterView<?> parent, View v, int position, long id) {
		if ((Integer)position != 0) {
			courseMgr = new CourseMgr(session, this);
			course = courseMgr.readCourse(courses.get(position));
			
			if (course == null) {
				errMsg(getString(R.string.err_course_no_exist));
			} else {
				loadDetail();
			}
		}
	}
	
	@Override
	public void onNothingSelected(AdapterView<?> arg0) {
	}
	
	/*
	 * Llegir cursos
	 */
	private void readCourses() {
    	coursesById.clear();
    	coursesByName.clear();
    	courses.clear();
    	
    	CourseMgr courseMgr = new CourseMgr(session, this);
    	Vector<Course> coursesV = courseMgr.readCourses();

    	Iterator<Course> itr = coursesV.iterator();
    	while (itr.hasNext()) {
          	Course course = itr.next();
          	coursesById.put(course.getIdNumber(), course);
          	coursesByName.put(course.getShortName(), course.getIdNumber());
        	courses.add(course.getShortName());
    	}
    	courses.add("");
    	Collections.sort(courses);
    }
	
	/*
	 * Mostra error
	 */
	private void errMsg(String msg) {
    	AlertDialog.Builder alertbox = new AlertDialog.Builder(this);
		alertbox.setMessage(msg);
		alertbox.setNeutralButton(R.string.ok, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface arg0, int arg1) {}
		});
		alertbox.show();
    }
    
	/*
	 * Carrega detall curs
	 */
	private void loadDetail(){
		edit_short.setText(course.getShortName());
		edit_cat.setText(course.getCategory());
		edit_full.setText(course.getFullName());
		edit_summary.setText(course.getSummary());
		changeStatus(BODY);
    }
	
	/*
	 * Neteja camps
	 */
	private void cleanFields(){
    	spin_short.setSelection(0);
    	edit_short.setText("");
    	edit_full.setText("");
    	edit_summary.setText("");
    }
    
	/*
	 * Canvi estat pantalla
	 */
    private void changeStatus(int status) {
    	this.status = status;
    	switch (status) {
		case INITIAL:
			readCourses();
			aAdapterShort.notifyDataSetChanged();
			cleanFields();
    		layout_body.setVisibility(View.GONE);
            spin_short.setVisibility(View.VISIBLE);
            break;
		case BODY:
      		layout_body.setVisibility(View.VISIBLE);
            spin_short.setVisibility(View.GONE);
            edit_short.requestFocus();
            break;
    	}
    }
}