/**
 * @Author Manel Herrera - UOC - PFC
 * @Date 2011/Q4 - V1.0
 *
 * Classe que interacciona amb llista de cursos
 */

package edu.manelherrera.pfc.pre;

import java.util.Collections;
import java.util.Vector;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ImageButton;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.AdapterView.OnItemClickListener;
import edu.manelherrera.pfc.bus.CourseMgr;
import edu.manelherrera.pfc.cla.Course;
import edu.manelherrera.pfc.cla.Session;
import edu.manelherrera.pfc.pre.utils.SortCoursesByCategory;
import edu.manelherrera.pfc.pre.utils.SortCoursesByName;

public class CourseListAct extends Activity implements OnClickListener{
	
	// Declaracions
	 private static final int NAME = 0;
	 private static final int CATEGORY = 1;
	 
	 private int order = NAME;
	 
	 private ImageButton button_sort;
	 private TextView label_usrDesc;
	 private TextView label_list_tit; 
	 private TextView label_courseListEmpty; 
	 private ListView listView_courses;
	 
	 private Vector<Course> courses; 
	 private Session session;
	 private Context context = this;
	 private CourseListAdapter courseListAdapter;
	
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
    	super.onCreate(savedInstanceState);
        setContentView(R.layout.course_list);
        
        //Inicialitzacions
        label_usrDesc = (TextView) findViewById(R.id.label_usrname);
        label_list_tit = (TextView) findViewById(R.id.label_list_tit);
        label_courseListEmpty = (TextView) findViewById(R.id.label_course_list_empty);
        listView_courses = (ListView) findViewById(R.id.listview_courses);
        button_sort = (ImageButton) findViewById(R.id.button_sort);
        
        button_sort.setOnClickListener(this); 
        listView_courses.setOnItemClickListener (listener);
        
        Bundle b = getIntent().getExtras();
        session = (Session) b.getParcelable("session");
        
        label_usrDesc.setText(session.getUserDesc());
      
        //Llegeix cursos
        readCourses(); 
				 
	}
    
    /*
     * Si es clica un bot
     */
    @Override
    public void onClick(View v) {
    	switch (order) {
    	case NAME:
    		sort(CATEGORY);
    		break;
    	case CATEGORY:
    		sort(NAME);
    		break;
    	}
  	}
    
    /*
     * Es crea listener per l'adaptador de la llista
     */
    OnItemClickListener listener = new OnItemClickListener () {    
    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id){
    	Intent myIntent = new Intent(context , CourseDetailAct.class);
    	Bundle b = new Bundle(); 
    	
		b.putParcelable("session", session);
		b.putParcelable("course", courses.get(position));
		myIntent.putExtras(b); 
		
		startActivity(myIntent);
   		}
    };

    /*
     * Es crea un adaptador especial per la llista
     */
    public class CourseListAdapter extends ArrayAdapter<Course> {
    	private Vector<Course> items;   
    	public CourseListAdapter(Context context, int textViewResourceId, Vector<Course> items) { 
    		super(context, textViewResourceId, items); 
    		this.items = items; 
    	} 
    	
    	@Override
    	public View getView(int position, View convertView, ViewGroup parent) { 
    		View v = convertView; 
    		if (v == null) { 
    			LayoutInflater vi = (LayoutInflater)getSystemService(Context.LAYOUT_INFLATER_SERVICE); 
    			v = vi.inflate(R.layout.course_list_layout, null);
    		} 
    		Course o = items.get(position); 
    		if (o != null) {   
 
    			TextView short_name = (TextView) v.findViewById(R.id.short_name);
    			TextView category = (TextView) v.findViewById(R.id.category);
    			TextView full_name = (TextView) v.findViewById(R.id.full_name);
    			TextView summary = (TextView) v.findViewById(R.id.summary);
    			if (short_name != null) { 
    				short_name.setText(o.getShortName()); 
    			} 
    			if (category != null) { 
    				category.setText("(" + o.getCategory() + ")"); 
    			} 
    			if (full_name != null) { 
    				full_name.setText(o.getFullName()); 
    			} 
    			if (summary != null) { 
    				summary.setText(summaryCut(o.getSummary())); 
    			} 
    		} 
    		
    		return v; 
    	} 
    } 
    
    /*
     * Llegeix cursos
     */
    private void readCourses(){
    	courses = new Vector<Course>();
        CourseMgr courseMgr = new CourseMgr(session, this);
		courses = courseMgr.readCourses();
		//courses.clear();
		
		if (courses.size() == 0) {
			label_courseListEmpty.setVisibility(View.VISIBLE);
			button_sort.setVisibility(View.GONE);
			label_list_tit.setVisibility(View.GONE);
		} else {
			label_courseListEmpty.setVisibility(View.GONE);
			button_sort.setVisibility(View.VISIBLE);
			label_list_tit.setVisibility(View.VISIBLE);
		}
		
		courseListAdapter = new CourseListAdapter(this, R.layout.layout,
				courses);
		listView_courses.setAdapter(courseListAdapter);
    	sort(order);
    }
    
    /*
     * Trunca resum curs
     */
    private String summaryCut(String summary) {
		if (summary.length() > 250) {
			return summary.substring(0, 250) + " ...";
		} else {
			return summary;
		}
	}
    
    /*
     * Ordenaci
     */
    private void sort(int order) {
    	this.order = order;
    	switch (order) {
		case NAME:
			label_list_tit.setText(getString(R.string.courses_by_name));
			Collections.sort(courses, new SortCoursesByName());
			courseListAdapter.notifyDataSetChanged();
            break;
		case CATEGORY:
			label_list_tit.setText(getString(R.string.courses_by_category));
			Collections.sort(courses, new SortCoursesByCategory());
			courseListAdapter.notifyDataSetChanged();
            break;
    	}

    }

}