/**
 * @Author Manel Herrera - UOC - PFC
 * @Date 2011/Q4 - V1.0
 *
 * Classe que interacciona amb el login de l'aplicaci
 */

package edu.manelherrera.pfc.pre;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.TextView;
import edu.manelherrera.pfc.bus.LoginMgr;
import edu.manelherrera.pfc.cla.Login;
import edu.manelherrera.pfc.cla.Session;

public class LoginAct extends Activity implements OnClickListener{
	
	//Declaracions
	private static final int LOGIN = 0;
	private static final int LOGOUT = 1;
	 
	private int status = LOGIN;
	
    private TextView label_usrDesc; 
	private ImageButton button_prefs;
	private ImageButton button_login;
	private Button button_logout;
	private EditText edit_usr; 
    private EditText edit_pwd; 
    private LinearLayout layout_login;
    private LinearLayout layout_logout;
    
    private SharedPreferences sharedPrefs;
    private Session session;
    private LoginMgr loginMgr;
		          
    private Context context;


	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.login);
                
        //Inicialitzacions
        label_usrDesc = (TextView) findViewById(R.id.label_usrname);
        
        edit_usr = (EditText)findViewById(R.id.edit_usr); 
        edit_pwd = (EditText)findViewById(R.id.edit_pwd); 
			           
        button_prefs = (ImageButton) findViewById(R.id.button_prefs);
        button_login = (ImageButton) findViewById(R.id.button_login);
        button_logout = (Button) findViewById(R.id.button_logout);
        layout_login = (LinearLayout) findViewById(R.id.layout_login);
        layout_logout = (LinearLayout) findViewById(R.id.layout_logout);
        
        button_prefs.setOnClickListener(this);
        button_login.setOnClickListener(this);
        button_logout.setOnClickListener(this);
        
        sharedPrefs = getSharedPreferences("moodlePrefs", MODE_PRIVATE);
        context = this;
        
    } 
    
    @Override
    public void onDestroy() {
    	super.onDestroy();
    	logout();
    }
    
    @Override
    public void onResume() {
    	super.onRestart();
    	changeStatus(status);
    }
    
    /*
     * Si es clica un boto
     */
    @Override
    public void onClick(View v) {
   	 	switch (v.getId()) {
   	 	//Login
		case R.id.button_login:
		
			String usr = edit_usr.getText().toString().trim();
	        String pwd = edit_pwd.getText().toString().trim();
	        
	        loginMgr = new LoginMgr(sharedPrefs, context);
	        Login login = new Login(usr, pwd, ""); 
	        String res = loginMgr.validateUserData(login);
	     
	        if (res == null) {
		        session = loginMgr.Login(login);
		       
		        if (session != null){
		        	Intent myIntent = new Intent(context , MainAct.class);
		        	
		        	Bundle b = new Bundle(); 
		        	b.putParcelable("session", session);
		        	myIntent.putExtras(b); 
	
		        	label_usrDesc.setText(session.getUserDesc());
		        	status = LOGOUT;

		            startActivity(myIntent);
		        } else {
		        	errMsg(getString(R.string.err_login)); 
		        }
	        } else {
	        	errMsg(res);
	        }
	        	        
	        cleanFields();
        
			break;
			
		//Logout
		case R.id.button_logout:
			logout();
			break;
			
		//Preferencies
		case R.id.button_prefs:
			Intent myIntent = new Intent(this , PrefsAct.class);
			startActivity(myIntent);
			break;
		}
  	}
    
    /*
     * Fa logout
     */
    private void logout() {
    	if (session != null) {
        	loginMgr.Logout(session);
        	session = null;
    	}
    	changeStatus(LOGIN);
    }
    
    /*
     * Mostra error
     */
    private void errMsg(String msg) {
    	AlertDialog.Builder alertbox = new AlertDialog.Builder(this);
		alertbox.setMessage(msg);
		alertbox.setNeutralButton(getString(R.string.ok), new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface arg0, int arg1) {}
		});
		alertbox.show();
    }
    
    /*
     * Neteja camps
     */
    private void cleanFields(){
    	edit_usr.setText("");
        edit_pwd.setText("");
        edit_usr.requestFocus();
    }
    
    /*
     * Canvi estatus de pantalla
     */
    private void changeStatus(int status) {
    	this.status = status;
    	switch (this.status) {
		case LOGIN:
			cleanFields();
			layout_login.setVisibility(View.VISIBLE);
			layout_logout.setVisibility(View.GONE);
            edit_usr.setText("");
            edit_pwd.setText("");
            break;
		case LOGOUT:
			layout_login.setVisibility(View.GONE);
			layout_logout.setVisibility(View.VISIBLE);
            break;
    	}
    }
    
	
}