/**
 * @Author Manel Herrera - UOC - PFC
 * @Date 2011/Q4 - V1.0
 *
 * Classe que interacciona amb el menu principal
 */
package edu.manelherrera.pfc.pre;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ImageButton;
import android.widget.TextView;
import edu.manelherrera.pfc.cla.Session;

public class MainAct extends Activity implements OnClickListener{
	
	//Declaracions
	private static final int CREATE_USER = 0;
	private static final int AMEND_USER = 1;
	private static final int CREATE_COURSE = 2;
	private static final int AMEND_COURSE = 3;
	private static final int LIST_COURSE = 4;
	private static final int DETAIL_COURSE = 5;
	private static final int LIST_ROLE = 6;
	private static final int CREATE_ROLE = 7;
	private static final int DELETE_ROLE = 8;
	
	private static final int USERS = 0;
	private static final int COURSES = 1;
	private static final int ROLES = 2;
	
	private int menuOption;
	
    private TextView label_usrDesc; 
    private ImageButton button_users;
    private ImageButton button_courses;
    private ImageButton button_enrol;
    
    private Session session;
	
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        //Inicialitzacions
        label_usrDesc = (TextView) findViewById(R.id.label_usrname);
        button_users = (ImageButton) findViewById(R.id.button_users);
        button_courses = (ImageButton) findViewById(R.id.button_courses);
        button_enrol = (ImageButton) findViewById(R.id.button_enrol);
        button_users.setOnClickListener(this);	
        button_courses.setOnClickListener(this); 	
        button_enrol.setOnClickListener(this); 	
                
        Bundle b = getIntent().getExtras();
        session = (Session) b.getParcelable("session"); 
               
        label_usrDesc.setText(session.getUserDesc());

    }	
    
    /*
     * Si es clica un bot
     */
    @Override
    public void onClick(View v) {
    	switch (v.getId()) {
		case R.id.button_users:
			menuOption = USERS;
			openOptionsMenu();
			break;
		case R.id.button_courses:
			menuOption = COURSES;
			openOptionsMenu();
			break;
		case R.id.button_enrol:
			menuOption = ROLES;
			openOptionsMenu();
			break;
		}
  	}
    
    @Override
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		return true; 
	}
    
    /*
     * mostra menus d'opcions segons bot clicat
     */
    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
    	menu.clear();
    	if (menuOption == USERS) {
			menu.add(Menu.NONE, CREATE_USER, 0, getString(R.string.menu_user_create));
			menu.add(Menu.NONE, AMEND_USER, 1, getString(R.string.menu_user_amend));
		}
    	else if (menuOption == COURSES){
			menu.add(Menu.NONE, CREATE_COURSE, 0, getString(R.string.menu_course_create));
			menu.add(Menu.NONE, AMEND_COURSE, 1, getString(R.string.menu_course_amend));
			menu.add(Menu.NONE, LIST_COURSE, 2, getString(R.string.menu_course_list));
			menu.add(Menu.NONE, DETAIL_COURSE, 3, getString(R.string.menu_course_detail));
		}
    	else if (menuOption == ROLES){
			menu.add(Menu.NONE, LIST_ROLE, 0, getString(R.string.menu_role_list));
			menu.add(Menu.NONE, CREATE_ROLE, 1, getString(R.string.menu_role_create));
			menu.add(Menu.NONE, DELETE_ROLE, 2, getString(R.string.menu_role_delete));
		}

		return true;
    }
    
    /*
     * Crida a l'acci de men que correspongui
     */
    @Override
	public boolean onOptionsItemSelected(MenuItem item) {
    	Intent myIntent;
    	Bundle b = new Bundle(); 
    	
    	switch (item.getItemId()) {
    	//Crea usuari
    	case CREATE_USER:
    		myIntent = new Intent(this , UserCreateAct.class);
					
    		b.putParcelable("session", session);
    		myIntent.putExtras(b); 
    		
			startActivity(myIntent);
			
			break;
	    //Modifica usuari
		case AMEND_USER:
			myIntent = new Intent(this , UserAmendAct.class);
			
			b.putParcelable("session", session);
			myIntent.putExtras(b); 
			
			startActivity(myIntent);
			
			break;
		//Crea curs
		case CREATE_COURSE:
    		myIntent = new Intent(this , CourseCreateAct.class);
					
    		b.putParcelable("session", session);
    		myIntent.putExtras(b); 
    		
			startActivity(myIntent);
			
			break;
		
		//Modifica curs
		case AMEND_COURSE:
			myIntent = new Intent(this , CourseAmendAct.class);
			
			b.putParcelable("session", session);
			myIntent.putExtras(b); 
			
			startActivity(myIntent);
			
			break;
		//Llista curs
		case LIST_COURSE:
    		myIntent = new Intent(this , CourseListAct.class);
					
    		b.putParcelable("session", session);
    		myIntent.putExtras(b); 
    		
			startActivity(myIntent);
			
			break;
		//Fitxa curs
		case DETAIL_COURSE:
			myIntent = new Intent(this , CourseDetailAct.class);
			
			b.putParcelable("session", session);
			myIntent.putExtras(b); 
			
			startActivity(myIntent);
			
			break;
		
		//Llista matricula
		case LIST_ROLE:
    		myIntent = new Intent(this , RoleListAct.class);
					
    		b.putParcelable("session", session);
    		myIntent.putExtras(b); 
    		
			startActivity(myIntent);
			
			break;
		//Crea matrcula
		case CREATE_ROLE:
			myIntent = new Intent(this , RoleCreateAct.class);
			
			b.putParcelable("session", session);
			myIntent.putExtras(b); 
			
			startActivity(myIntent);
			
			break;
		//Elimina matricula
		case DELETE_ROLE:
			myIntent = new Intent(this , RoleDeleteAct.class);
			
			b.putParcelable("session", session);
			myIntent.putExtras(b); 
			
			startActivity(myIntent);
			
			break;
		}
		return true;
	}
}