/**
 * @Author Manel Herrera - UOC - PFC
 * @Date 2011/Q4 - V1.0
 *
 * Classe que interacciona amb l'eliminaci de matrcules
 */

package edu.manelherrera.pfc.pre;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemSelectedListener;
import edu.manelherrera.pfc.bus.CourseMgr;
import edu.manelherrera.pfc.bus.RoleMgr;
import edu.manelherrera.pfc.bus.UserMgr;
import edu.manelherrera.pfc.cla.Course;
import edu.manelherrera.pfc.cla.Session;
import edu.manelherrera.pfc.cla.User;

public class RoleDeleteAct extends Activity implements OnClickListener, 
		OnItemSelectedListener {
	
	//Declaracions
	 private static final int INITIAL = 0;
	 private static final int BODY = 1;
	 
	 private int status;
	 
	 private Course course;
	 private User user;
	 
	 private ImageButton button_del;
	 private ImageButton button_close;
	 private ImageButton button_search;	
	 private TextView label_usrDesc;
	 private TextView label_short;
	 private TextView label_courseListEmpty; 
	 private EditText edit_usr; 
	 private Spinner spin_short; 
	 private LinearLayout layout_body;
	 
	 private ArrayAdapter<String> aAdapterShort;
	 private List<String> courses;
	 
	 private Session session;
	
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
    	super.onCreate(savedInstanceState);
        setContentView(R.layout.role_delete);
        
        //Inicialitzacions
        edit_usr = (EditText)findViewById(R.id.edit_usr); 
        label_usrDesc = (TextView) findViewById(R.id.label_usrname);
        label_short = (TextView) findViewById(R.id.label_short);
        label_courseListEmpty = (TextView) findViewById(R.id.label_course_list_empty);
        button_del = (ImageButton) findViewById(R.id.button_del);
        button_close = (ImageButton) findViewById(R.id.button_close);
        button_search = (ImageButton) findViewById(R.id.button_search);
        spin_short = (Spinner)findViewById(R.id.spin_short); 
        layout_body = (LinearLayout) findViewById(R.id.layout_body);
        
        spin_short.setOnItemSelectedListener(this);
        button_del.setOnClickListener(this);
        button_close.setOnClickListener(this);
        button_search.setOnClickListener(this);
        
        Bundle b = getIntent().getExtras();
        session = (Session) b.getParcelable("session");
        
        label_usrDesc.setText(session.getUserDesc());
        
        courses = new ArrayList<String>();

        aAdapterShort = new ArrayAdapter<String>(this,
				R.layout.layout_spinner, courses);
        aAdapterShort.setDropDownViewResource(R.layout.simple_spinner_dropdown_item);
		spin_short.setAdapter(aAdapterShort);
        
        changeStatus(INITIAL);
				 
	}
	
    /*
     * Si es clica un bot
     */
    @Override
    public void onClick(View v) {
    	
    	switch (v.getId()) {
    	//Cerca usuari
    	case R.id.button_search:
    		String usr = edit_usr.getText().toString().trim();
    		UserMgr userMgr = new UserMgr(session, this);
    		user = userMgr.readUser(usr);
    		if (user == null) {
    			errMsg(getString(R.string.err_usr_no_exist));
    			changeStatus(INITIAL);
    		} else {
    	        readEnrolments();
    	        aAdapterShort.notifyDataSetChanged();
    	        changeStatus(BODY);
    		}
    		break;
    	//Elimina matricula
    	case R.id.button_del: 
    		
			AlertDialog.Builder alertbox = new AlertDialog.Builder(this);
			alertbox.setCancelable(false);
			alertbox.setMessage(getString(R.string.role_delete_conf));
			alertbox.setPositiveButton(R.string.yes, new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface arg0, int arg1) {
					delete();
				}
			});alertbox.setNegativeButton(R.string.no, new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog, int arg1) {
					dialog.cancel();
				}
			});
			
			alertbox.show();
			break;
		//Tanca
    	case R.id.button_close:    
    		switch (status) {
			case INITIAL:
				this.finish();
				break;
			case BODY:
				changeStatus(INITIAL);
				break;
			}
			break;
    	}
    }

    /*
     * Si se selecciona un element de la llista
     */
    @Override
	public void onItemSelected(AdapterView<?> parent, View v, int position, long id) {

		if ((Integer)position != 0) {
			CourseMgr courseMgr = new CourseMgr(session, this);
			course = courseMgr.readCourse(courses.get(position));

			if (course == null) {
				errMsg(getString(R.string.err_course_no_exist));
			}
		}
	}
	
	@Override
	public void onNothingSelected(AdapterView<?> arg0) {

	}
	
	/*
	 * LLegeix matrcules
	 */
	private void readEnrolments() {
		courses.clear();
		
		RoleMgr roleMgr = new RoleMgr(session, this);
		Vector<Course> coursesV = roleMgr.readEnrolmentsByUserName(user.getuserName());
	
		if (coursesV.size() == 0) {
			label_courseListEmpty.setVisibility(View.VISIBLE);
			label_short.setVisibility(View.GONE);
			spin_short.setVisibility(View.GONE);
			button_del.setVisibility(View.GONE);

		} else {
			label_courseListEmpty.setVisibility(View.GONE);
			label_short.setVisibility(View.VISIBLE);
			spin_short.setVisibility(View.VISIBLE);
			button_del.setVisibility(View.VISIBLE);
			
			Iterator<Course> itr = coursesV.iterator();
			while (itr.hasNext()) {
		      	Course course = itr.next();
		    	courses.add(course.getShortName());
			}
			courses.add("");
			Collections.sort(courses);
		}
		
	}
    
	/*
	 * Canvi estatus pantalla
	 */
   private void changeStatus(int status) {
    	this.status = status;
    	switch (this.status) {
			case INITIAL:
			edit_usr.setText("");
			user = null;
			course = null;
			edit_usr.setEnabled(true);
			button_search.setVisibility(View.VISIBLE);
      		layout_body.setVisibility(View.GONE);
      		label_courseListEmpty.setVisibility(View.GONE);
	        break;
		case BODY:
			spin_short.setSelection(0);
			edit_usr.setEnabled(false);
			button_search.setVisibility(View.GONE);
      		layout_body.setVisibility(View.VISIBLE);
	        break;
		}
	}
	
   /*
    * Mostra error
    */
	private void errMsg(String msg) {
		AlertDialog.Builder alertbox = new AlertDialog.Builder(this);
		alertbox.setMessage(msg);
		alertbox.setNeutralButton(R.string.ok, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface arg0, int arg1) {}
		});
		alertbox.show();
	}
    
	/*
	 * Elimina matrcula
	 */
	private void delete() {
		RoleMgr roleMgr = new RoleMgr(session, this);
		String res = roleMgr.deleteEnrolment(course, user);
		if (res.compareTo("") == 0) {
	   		Toast.makeText(this, getString(R.string.role_deleted), Toast.LENGTH_LONG).show();
	   		changeStatus(INITIAL);
	   	} else {
	   		errMsg(res);
	   	}
	}
}