/**
 * @Author Manel Herrera - UOC - PFC
 * @Date 2011/Q4 - V1.0
 *
 * Classe que interacciona amb llista de matricules
 */

package edu.manelherrera.pfc.pre;

import java.util.Collections;
import java.util.Vector;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.AdapterView.OnItemClickListener;
import edu.manelherrera.pfc.bus.RoleMgr;
import edu.manelherrera.pfc.bus.UserMgr;
import edu.manelherrera.pfc.cla.Course;
import edu.manelherrera.pfc.cla.Session;
import edu.manelherrera.pfc.cla.User;
import edu.manelherrera.pfc.pre.utils.SortCoursesByCategory;
import edu.manelherrera.pfc.pre.utils.SortCoursesByName;

public class RoleListAct extends Activity implements OnClickListener{
	
	//Declaracions
	 private static final int NAME = 0;
	 private static final int CATEGORY = 1;
	 
	 private static final int INITIAL = 0;
	 private static final int BODY = 1;
	 
	 private int status;
	 	 
	 private int order = NAME;
	 
	 private ImageButton button_sort;
	 private ImageButton button_search;	
	 private TextView label_usrDesc;
	 private TextView label_list_tit; 
	 private TextView label_courseListEmpty; 
	 private ListView listView_courses;
	 private EditText edit_usr; 
	 private LinearLayout layout_body;
	 
	 private Vector<Course> courses; 
	 private Session session;
	 private Context context = this;
	 private CourseListAdapter courseListAdapter;
	
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
    	super.onCreate(savedInstanceState);
        setContentView(R.layout.role_list);
        
        //Inicialitzacions
        edit_usr = (EditText)findViewById(R.id.edit_usr); 
        label_usrDesc = (TextView) findViewById(R.id.label_usrname);
        label_list_tit = (TextView) findViewById(R.id.label_list_tit);
        label_courseListEmpty = (TextView) findViewById(R.id.label_course_list_empty);
        listView_courses = (ListView) findViewById(R.id.listview_courses);
        button_sort = (ImageButton) findViewById(R.id.button_sort);
        button_search = (ImageButton) findViewById(R.id.button_search);
        layout_body = (LinearLayout) findViewById(R.id.layout_body);
        
        button_sort.setOnClickListener(this);
        button_search.setOnClickListener(this);
        listView_courses.setOnItemClickListener (listener);
        
        Bundle b = getIntent().getExtras();
        session = (Session) b.getParcelable("session");
        
        label_usrDesc.setText(session.getUserDesc());
        
        changeStatus(INITIAL);
				 
	}
	
    /*
     * Si es clica un bot
     */
    @Override
    public void onClick(View v) {
    	
    	switch (v.getId()) {
    	//cerca usuari
    	case R.id.button_search:
    		String usr = edit_usr.getText().toString().trim();
    		UserMgr userMgr = new UserMgr(session, this);
    		User user = userMgr.readUser(usr);
    		if (user == null) {
    			errMsg(getString(R.string.err_usr_no_exist));
    			if (courses != null) {
    				changeStatus(INITIAL);
    				courses.clear();
    				courseListAdapter.notifyDataSetChanged();
    			}
    		} else {
    			readCourses(usr);
    			changeStatus(BODY);
    		}
    		break;
    	//Ordena
    	case R.id.button_sort:    
    		switch (order) {
    		case NAME:
    			sort(CATEGORY);
    			break;
    		case CATEGORY:
    			sort(NAME);
    			break;
    		}
    	}
  	}
    
    /*
     * Crea un listener per l'adapter
     */
    OnItemClickListener listener = new OnItemClickListener () {    
    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id){
    	Intent myIntent = new Intent(context , CourseDetailAct.class);
    	Bundle b = new Bundle(); 
    	
		b.putParcelable("session", session);
		b.putParcelable("course", courses.get(position));
		myIntent.putExtras(b); 
		
		startActivity(myIntent);
   		}
    };

    /*
     * Crea un adapter especial per la llista
     */
    public class CourseListAdapter extends ArrayAdapter<Course> {
    	private Vector<Course> items;   
    	public CourseListAdapter(Context context, int textViewResourceId, Vector<Course> items) { 
    		super(context, textViewResourceId, items); 
    		this.items = items; 
    	} 
    	
    	@Override
    	public View getView(int position, View convertView, ViewGroup parent) { 
    		View v = convertView; 
    		if (v == null) { 
    			LayoutInflater vi = (LayoutInflater)getSystemService(Context.LAYOUT_INFLATER_SERVICE); 
    			v = vi.inflate(R.layout.course_list_layout, null);
    		} 
    		Course o = items.get(position); 
    		if (o != null) {   
 
    			TextView short_name = (TextView) v.findViewById(R.id.short_name);
    			TextView category = (TextView) v.findViewById(R.id.category);
    			TextView full_name = (TextView) v.findViewById(R.id.full_name);
    			TextView summary = (TextView) v.findViewById(R.id.summary);
    			if (short_name != null) { 
    				short_name.setText(o.getShortName()); 
    			} 
    			if (category != null) { 
    				category.setText("(" + o.getCategory() + ")"); 
    			} 
    			if (full_name != null) { 
    				full_name.setText(o.getFullName()); 
    			} 
    			if (summary != null) { 
    				summary.setText(summaryCut(o.getSummary())); 
    			} 
    		} 
    		
    		return v; 
    	} 
    } 
    
    //Llegeix cursos
    private void readCourses(String usr){
    	courses = new Vector<Course>();
        RoleMgr roleMgr = new RoleMgr(session, this);
		courses = roleMgr.readEnrolmentsByUserName(usr);
		//courses.clear();
		
		if (courses.size() == 0) {
			label_courseListEmpty.setVisibility(View.VISIBLE);
			button_sort.setVisibility(View.GONE);
			label_list_tit.setVisibility(View.GONE);
		} else {
			label_courseListEmpty.setVisibility(View.GONE);
			button_sort.setVisibility(View.VISIBLE);
			label_list_tit.setVisibility(View.VISIBLE);
		}
		
		courseListAdapter = new CourseListAdapter(this, R.layout.layout,
				courses);
		listView_courses.setAdapter(courseListAdapter);
    	sort(order);
    }
    
    /*
     * Trunca el resum del curs
     */
    private String summaryCut(String summary) {
		if (summary.length() > 250) {
			return summary.substring(0, 250) + " ...";
		} else {
			return summary;
		}
	}
    
    /*
     * Ordena
     */
    private void sort(int order) {
    	this.order = order;
    	switch (order) {
		case NAME:
			label_list_tit.setText(getString(R.string.courses_by_name));
			Collections.sort(courses, new SortCoursesByName());
			courseListAdapter.notifyDataSetChanged();
            break;
		case CATEGORY:
			label_list_tit.setText(getString(R.string.courses_by_category));
			Collections.sort(courses, new SortCoursesByCategory());
			courseListAdapter.notifyDataSetChanged();
            break;
    	}

    }
    
    /*
     * Canvi estatus pantalla
     */
    private void changeStatus(int status) {
    	this.status = status;
    	switch (this.status) {
		case INITIAL:
			edit_usr.setText("");
    		layout_body.setVisibility(View.GONE);
            break;
		case BODY:
      		layout_body.setVisibility(View.VISIBLE);
            break;
    	}
    }

    /*
     * Mostra missatge
     */
    private void errMsg(String msg) {
    	AlertDialog.Builder alertbox = new AlertDialog.Builder(this);
		alertbox.setMessage(msg);
		alertbox.setNeutralButton(R.string.ok, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface arg0, int arg1) {}
		});
		alertbox.show();
    }
    
}