/**
 * @Author Manel Herrera - UOC - PFC
 * @Date 2011/Q4 - V1.0
 *
 * Classe que interacciona amb modificaci d'usuaris
 */

package edu.manelherrera.pfc.pre;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;
import edu.manelherrera.pfc.bus.UserMgr;
import edu.manelherrera.pfc.cla.Session;
import edu.manelherrera.pfc.cla.User;

public class UserAmendAct extends Activity implements OnClickListener{
	
	//Declaracions
	 private static final int INITIAL = 0;
	 private static final int BODY = 1;
	 
	 private int status;
	 
	 private String usr;
	 private String pwd;
	 private UserMgr userMgr;
	 private User user = null;
	    
	 private TextView label_usrDesc; 
	 private EditText edit_usr; 
	 private EditText edit_pwd; 
	 private EditText edit_name; 
	 private EditText edit_surname; 
	 private EditText edit_email; 
	 private ImageButton button_del;
	 private ImageButton button_close;
	 private ImageButton button_save;
	 private ImageButton button_search;	
	 private LinearLayout layout_body;
	 
	 private Session session;
	
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.user_amend);
        
        //Inicialitzacions
        edit_usr = (EditText)findViewById(R.id.edit_usr); 
        edit_pwd = (EditText)findViewById(R.id.edit_pwd);
        edit_name = (EditText)findViewById(R.id.edit_name); 
        edit_surname = (EditText)findViewById(R.id.edit_surname); 
        edit_email = (EditText)findViewById(R.id.edit_email); 
        label_usrDesc = (TextView) findViewById(R.id.label_usrname);
        button_del = (ImageButton) findViewById(R.id.button_del);
        button_close = (ImageButton) findViewById(R.id.button_close);
        button_save = (ImageButton) findViewById(R.id.button_save);
        button_search = (ImageButton) findViewById(R.id.button_search);
        layout_body = (LinearLayout) findViewById(R.id.layout_body);
      
        button_del.setOnClickListener(this);
        button_close.setOnClickListener(this);
        button_save.setOnClickListener(this);
        button_search.setOnClickListener(this);	
        
        Bundle b = getIntent().getExtras();
        session = (Session) b.getParcelable("session"); 
     
        label_usrDesc.setText(session.getUserDesc());
        
        changeStatus(INITIAL);
    }
    
    /*
     * Si es clica un bot
     */
    @Override
    public void onClick(View v) {
        	
    	switch (v.getId()) {
    	//Cerca usuari
    	case R.id.button_search:
			usr = edit_usr.getText().toString().trim();
			userMgr = new UserMgr(session, this);
			user = userMgr.readUser(usr);
			if (user == null) {
				errMsg(getString(R.string.err_usr_no_exist));
			} else {
				edit_name.setText(user.getfirstName());
				edit_surname.setText(user.getlastName());
				edit_email.setText(user.geteMail());
				changeStatus(BODY);
			}
		    break;
		//Salva usuari
		case R.id.button_save:
			String fName = edit_name.getText().toString().trim();
			String lName = edit_surname.getText().toString().trim();
			String eMail = edit_email.getText().toString().trim();
			        
			user = new User(usr, "", fName, lName, eMail);
			userMgr = new UserMgr(session, this);
			String res = userMgr.amendUser(user);
			if (res == null) {
				Toast.makeText(this, getString(R.string.user_updated), Toast.LENGTH_LONG).show();
				changeStatus(INITIAL);
			} else {
				errMsg(res);
			}
			break;
			
	    //Tanca
		case R.id.button_close:
			switch (status) {
			case INITIAL:
				this.finish();
				break;
			case BODY:
				changeStatus(INITIAL);
				break;
			}
			break;
		//Elimina
		case R.id.button_del:
			
			AlertDialog.Builder alertbox = new AlertDialog.Builder(this);
			alertbox.setCancelable(false);
			alertbox.setMessage(getString(R.string.user_delete_conf));
			alertbox.setPositiveButton(R.string.yes, new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface arg0, int arg1) {
					delete();
				}
			});alertbox.setNegativeButton(R.string.no, new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog, int arg1) {
					dialog.cancel();
				}
			});
			
			alertbox.show();
			
			break;
		}
    }
    
    /*
     * Mostra missatge
     */
    private void errMsg(String msg) {
    	AlertDialog.Builder alertbox = new AlertDialog.Builder(this);
		alertbox.setMessage(msg);
		alertbox.setNeutralButton(R.string.ok, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface arg0, int arg1) {}
		});
		alertbox.show();
    }
    
    /*
     * Elimina usuari
     */
    private void delete() {
    	String fName = edit_name.getText().toString().trim();
	    String lName = edit_surname.getText().toString().trim();
	    String eMail = edit_email.getText().toString().trim();
	        
	   	user = new User(usr, pwd, fName, lName, eMail);
	   	userMgr = new UserMgr(session, this);
	   	String res = userMgr.deleteUser(user);
	   	if (res == null) {
	   		Toast.makeText(this, getString(R.string.user_deleted), Toast.LENGTH_LONG).show();
	   		changeStatus(INITIAL);
	   	} else {
	   		errMsg(res);
	   	}
    }
    
    /*
     * Neteja camps
     */
    private void cleanFields(){
    	edit_usr.setText("");
        edit_pwd.setText("");
        edit_name.setText("");
        edit_surname.setText("");
        edit_email.setText("");
        edit_usr.requestFocus();
    }
    
    /*
     * Canvi estatus pantalla
     */
    private void changeStatus(int status) {
    	this.status = status;
    	switch (status) {
		case INITIAL:
			cleanFields();
			button_search.setVisibility(View.VISIBLE);
    		layout_body.setVisibility(View.GONE);
            button_del.setVisibility(View.GONE);
            edit_usr.setEnabled(true);  
            edit_usr.requestFocus();
            break;
		case BODY:
			button_search.setVisibility(View.GONE);
      		layout_body.setVisibility(View.VISIBLE);
            button_del.setVisibility(View.VISIBLE);
            edit_usr.setEnabled(false); 
            edit_name.requestFocus();
            break;
    	}

    }
}